/**
 *  Copyright (C) 2009 Michel Vinicius de Melo Euzebio e Emerson Ribeiro de Mello
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 *  Copyright (C) 2009 Michel Vinicius de Melo Euzebio e Emerson Ribeiro de Mello
 *
 *  Este programa é software livre; você pode redistribuí-lo e/ou
 *  modificá-lo sob os termos da Licença Pública Geral GNU, conforme
 *  publicada pela Free Software Foundation; tanto a versão 2 da
 *  Licença como (a seu critério) qualquer versão mais nova.
 *
 *  Este programa é distribuído na expectativa de ser útil, mas SEM
 *  QUALQUER GARANTIA; sem mesmo a garantia implícita de
 *  COMERCIALIZAÇÃO ou de ADEQUAÇÃO A QUALQUER PROPÓSITO EM
 *  PARTICULAR. Consulte a Licença Pública Geral GNU para obter mais
 *  detalhes.
 *
 *  Você deve ter recebido uma cópia da Licença Pública Geral GNU
 *  junto com este programa; se não, escreva para a Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
 *  02111-1307, USA.
*/

package app.dl;

import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.OutputStreamWriter;
import java.util.Scanner;

import android.app.Activity;
import android.os.Bundle;
import android.view.KeyEvent;
import android.view.View;
import android.view.View.OnClickListener;
import android.view.View.OnKeyListener;
import android.widget.Button;
import android.widget.EditText;
import android.widget.RadioButton;
import android.widget.Toast;

/**
 * Classe que monta e gerencia a tela de configurações.
 */
public class Configuracoes extends Activity implements OnClickListener, OnKeyListener{
	
	EditText usuario;
	EditText senha;
	EditText endereco;
	EditText porta;
	RadioButton httpButton;
	RadioButton httpsButton;
	
	private Conexao conexao;
	
	@Override
    public void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.login);
        
        usuario = (EditText)findViewById(R.id.etUsuario);
        senha = (EditText)findViewById(R.id.etSenha);
        endereco = (EditText)findViewById(R.id.etEndereco);
        porta = (EditText)findViewById(R.id.etPorta);
        httpButton = (RadioButton)findViewById(R.id.httpPorta);
        httpsButton = (RadioButton)findViewById(R.id.httpsPorta);
               
        porta.setOnKeyListener(this);
        httpButton.setOnClickListener(this);
        httpsButton.setOnClickListener(this);
        
        Button salvar = (Button)findViewById(R.id.salvar);     
        salvar.setOnClickListener(this);
        
        recuperarDados();

	}

	@Override
	public void onClick(View v) {
		
		switch(v.getId()){
		
		case R.id.salvar: salvarDados();						  
						  break;
			
		case R.id.httpPorta: porta.setText("8080"); break;
			
		case R.id.httpsPorta: porta.setText("8181");
			
		}		
		
	}

	/**
	 * Grava as informações inseridas pelo usuário.
	 */
	private void salvarDados() {
		String u = usuario.getText().toString();
		String s = senha.getText().toString();
		String e = endereco.getText().toString();
		String p = porta.getText().toString();
		
		if (!u.equals("") && !s.equals("") && !e.equals("") && !p.equals("")) {
			conexao = new Conexao(this, e, p, httpButton.isChecked());

			// Salva as informacoes no arquivo de configuracao
			try {
				FileOutputStream fOut = openFileOutput("conf.dat", MODE_PRIVATE);
				OutputStreamWriter osw = new OutputStreamWriter(fOut);

				osw.write(u + "\n" + s + "\n" + e + "\n" + p + "\n"
						+ httpButton.isChecked());

				osw.close();
				fOut.close();
				
				String response = conexao.enviaMensagem(Conexao.MSG_AUTENTICACAO, u, s, "0");
				
				if(response.equals(Conexao.MSG_ERRO)){
					Toast.makeText(this, "Erro de autenticação!", Toast.LENGTH_SHORT).show();
				}else if(response.equals("")){
					Toast.makeText(this, "Erro na conexão com o servidor!", Toast.LENGTH_SHORT).show();
				}else{
					this.setResult(Activity.RESULT_OK);
					this.finish();
				}

			} catch (IOException ioe) {
				ioe.printStackTrace();
			}
		}else{
			Toast.makeText(this, "Preencha todos os campos!", Toast.LENGTH_SHORT).show();
		}
	}
	
	/**
	 * Restaura informações da tela gravadas em arquivo.
	 */
	private void recuperarDados() {
		// Abre arquivo de configuracao para configurar atributos
		try {
			FileInputStream fIn = openFileInput("conf.dat");
			Scanner file_scan = new Scanner(fIn);

			if (file_scan.hasNext()) {
				usuario.setText(file_scan.nextLine());
			}
			if (file_scan.hasNext()) {
				senha.setText(file_scan.nextLine());
			}
			if (file_scan.hasNext()) {
				endereco.setText(file_scan.nextLine());
			}
			if (file_scan.hasNext()) {
				porta.setText(file_scan.nextLine());
			}
			if (file_scan.hasNext()) {
				if (file_scan.nextLine().equals("true")) {
					httpButton.setChecked(true);
				} else {
					httpsButton.setChecked(true);
				}
			}
			
			file_scan.close();
			fIn.close();

		} catch (IOException ioe) {
			ioe.printStackTrace();
			porta.setText("8080");
			httpButton.setChecked(true);
		}		
	}

	@Override
	public boolean onKey(View arg0, int arg1, KeyEvent arg2) {
		
		if(porta.getText().toString().equals("8080")){
			httpButton.setChecked(true);
		}else if(porta.getText().toString().equals("8181")){
			httpsButton.setChecked(true);
		}
		
		return false;
	}
	
	public boolean onKeyDown (int keyCode, KeyEvent event){
    	if(keyCode == KeyEvent.KEYCODE_BACK){
    		
    		this.setResult(Activity.RESULT_CANCELED);
    		finish();  	
    	}  	
    		
    	return super.onKeyDown(keyCode, event);  	
    }

}

