/**
 *  Copyright (C) 2009 Michel Vinicius de Melo Euzebio e Emerson Ribeiro de Mello
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 *  Copyright (C) 2009 Michel Vinicius de Melo Euzebio e Emerson Ribeiro de Mello
 *
 *  Este programa é software livre; você pode redistribuí-lo e/ou
 *  modificá-lo sob os termos da Licença Pública Geral GNU, conforme
 *  publicada pela Free Software Foundation; tanto a versão 2 da
 *  Licença como (a seu critério) qualquer versão mais nova.
 *
 *  Este programa é distribuído na expectativa de ser útil, mas SEM
 *  QUALQUER GARANTIA; sem mesmo a garantia implícita de
 *  COMERCIALIZAÇÃO ou de ADEQUAÇÃO A QUALQUER PROPÓSITO EM
 *  PARTICULAR. Consulte a Licença Pública Geral GNU para obter mais
 *  detalhes.
 *
 *  Você deve ter recebido uma cópia da Licença Pública Geral GNU
 *  junto com este programa; se não, escreva para a Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
 *  02111-1307, USA.
*/

package app.dl;

import java.util.ArrayList;

import android.content.Context;
import android.graphics.Color;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.BaseAdapter;
import android.widget.CheckBox;
import android.widget.SeekBar;
import android.widget.TextView;

/**
 * Classe que utiliza as listas com os nomes dos dispositivos e 
 * seus estados para montar a lista exibida pela ListView.
 */
public class LampadaAdapter extends BaseAdapter {
	
	private LayoutInflater mInflater;
	private Lampadas pai;
	
	private ArrayList<CheckBox> cboxes;
	private ArrayList<SeekBar> barras;
	private ArrayList<TextView> labels;
	private ArrayList<Integer> cores;

	public LampadaAdapter(Context context) {
		mInflater = LayoutInflater.from(context);
		pai = (Lampadas) context;	
		
		cboxes = new ArrayList<CheckBox>();
		barras = new ArrayList<SeekBar>();
		labels = new ArrayList<TextView>();
		cores = new ArrayList<Integer>();
	}

	/**
	 * Retorna a quantidade de posições exibidas pela ListView.
	 * @return quantidade de posições
	 */
	public int getCount() {
		return pai.getNomes().size();
	}

	public Object getItem(int position) {
		return position;
	}

	public long getItemId(int position) {
		return position;
	}

	/**
	 * Método invocado a cada atualização de tela que retorna a View 
	 * que será exibida em cada posição da ListView.
	 * @param position posição na qual será exibida a View retornada
	 * @param convertView View que será exibida na posição determinada pelo parametro position
	 * @param parent ListView na qual será exibida a lista de Views
	 * @return View que será exibida em uma determinada posição
	 */
	public View getView(int position, View convertView, ViewGroup parent) {
		ViewHolder holder;
		// se convertView está vazia  então cria os objetos a partir do XML inflado
		if (convertView == null) {
			
			convertView = mInflater.inflate(R.layout.lamp_lista, null);
			
			holder = new ViewHolder();
			holder.nomeLabel = (TextView)convertView.findViewById(R.id.nomeDisp);
			holder.interruptor = (CheckBox)convertView.findViewById(R.id.interruptor);
			holder.dimmer = (SeekBar)convertView.findViewById(R.id.dimmer);
						
			holder.interruptor.setOnClickListener(pai);
			holder.dimmer.setOnSeekBarChangeListener(pai);
			holder.nomeLabel.setOnTouchListener(pai);

			convertView.setTag(holder);
		} else {
			// se convertView não está vazia então converte ela para ViewHolder 
			holder = (ViewHolder) convertView.getTag();
		}
		
		// se a posição de convertView já existir nas listas de Views 
		// então substitui a view atual pela convertView  
		if(position < cboxes.size()){
			barras.set(position, holder.dimmer);
			cboxes.set(position, holder.interruptor);
			labels.set(position, holder.nomeLabel);	
			holder.nomeLabel.setTextColor(cores.get(position));
		}else{
			// se a posição não existir então adiciona uma nova 
			// posição nas listas de Views
			barras.add(holder.dimmer);
			cboxes.add(holder.interruptor);
			labels.add(holder.nomeLabel);
			cores.add(Color.WHITE);
		}
		
		// configura as Views com imagens, textos e etc.
			
		holder.nomeLabel.setText(pai.getNomes().get(position));			
		
		// converte valor de 0-255 em percentual (0-100)
		int estado = Integer.parseInt(pai.getEstados().get(position));
		estado = (estado*100)/255;
		
		pai.setImagem(holder.interruptor, estado);	
		holder.dimmer.setProgress(estado);
	
		return convertView;
	}
	
	/**
	 * @return lista de CheckBox que já foram exibidas na tela
	 */
	public ArrayList<CheckBox> getBoxes(){
		return cboxes;
	}
	
	/**
	 * @return lista de SeekBar que já foram exibidas na tela
	 */
	public ArrayList<SeekBar> getBarras(){
		return barras;
	}
	
	/**
	 * @return lista de TextView que já foram exibidas na tela
	 */
	public ArrayList<TextView> getLabels(){
		return labels;
	}
	
	/**
	 * @return lista de cores atuais das TextViews que já foram exibidas na tela
	 */
	public ArrayList<Integer> getCores(){
		return cores;
	}
	
	/**
	 * Classe que armazena Views inseridas em cada posição da ListView
	 */
	public class ViewHolder {
		TextView nomeLabel;
		CheckBox interruptor;
		SeekBar dimmer;
	}
}