/**
 *  Copyright (C) 2009 Michel Vinicius de Melo Euzebio e Emerson Ribeiro de Mello
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 *  Copyright (C) 2009 Michel Vinicius de Melo Euzebio e Emerson Ribeiro de Mello
 *
 *  Este programa é software livre; você pode redistribuí-lo e/ou
 *  modificá-lo sob os termos da Licença Pública Geral GNU, conforme
 *  publicada pela Free Software Foundation; tanto a versão 2 da
 *  Licença como (a seu critério) qualquer versão mais nova.
 *
 *  Este programa é distribuído na expectativa de ser útil, mas SEM
 *  QUALQUER GARANTIA; sem mesmo a garantia implícita de
 *  COMERCIALIZAÇÃO ou de ADEQUAÇÃO A QUALQUER PROPÓSITO EM
 *  PARTICULAR. Consulte a Licença Pública Geral GNU para obter mais
 *  detalhes.
 *
 *  Você deve ter recebido uma cópia da Licença Pública Geral GNU
 *  junto com este programa; se não, escreva para a Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
 *  02111-1307, USA.
*/

package app.dl;

import java.io.FileInputStream;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Scanner;

import android.app.Activity;
import android.app.AlertDialog;
import android.content.DialogInterface;
import android.content.Intent;
import android.graphics.Color;
import android.os.Bundle;
import android.view.KeyEvent;
import android.view.LayoutInflater;
import android.view.MenuItem;
import android.view.MotionEvent;
import android.view.View;
import android.view.View.OnClickListener;
import android.view.View.OnTouchListener;
import android.widget.CheckBox;
import android.widget.EditText;
import android.widget.ListView;
import android.widget.TextView;
import android.widget.Toast;
import android.widget.SeekBar;
import android.widget.SeekBar.OnSeekBarChangeListener;

/**
 * Classe responsável pela criação e gerenciamento da interface de controle dos dispositivos.
 */
public class Lampadas extends Activity implements OnSeekBarChangeListener, OnClickListener, OnTouchListener {

	
	private ArrayList<String> enderecos;
	private ArrayList<String> nomes;
	private ArrayList<String> estados;
	private ArrayList<String> disps;
	
	private ListView lista;
	private LampadaAdapter adapter;
	private Conexao conexao;
	private ArrayList<Integer> limiares;
	private boolean mudou;
	private int seekPosition;
	private String seekResposnse;
	
	private String usuario;
    private String senha;
    private String ip;
    private String porta;
    private boolean isHttp;
	
	public Lampadas(){
		nomes = new ArrayList<String>();
		estados = new ArrayList<String>();
		enderecos = new ArrayList<String>();
		disps = new ArrayList<String>();
				
		limiares = new ArrayList<Integer>();	
		mudou = false;
	}

	@Override
	public void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
		setContentView(R.layout.lampadas);
		
		// se conseguir recuperar as informações de conexão com o 
		// SAR então envia pedido de autenticação, se não conseguir então inicia a
		// tela de configurações
		if (setInfoServidor() == true) {

			conexao = new Conexao(this, ip, porta, isHttp);
			String response = conexao.enviaMensagem(Conexao.MSG_AUTENTICACAO, usuario, senha, "");

			if (response.equals(Conexao.MSG_ERRO)) {
				criarDialog(true);
			} else if (response.equals("")) {
				criarDialog(false);
			} else {
				lista = (ListView) findViewById(R.id.lampLista);
				this.updateAdapter();
			}
		}else{
			startActivity(new Intent(this, Configuracoes.class));
			finish();
		}
	}
	
	@Override
    public boolean  onCreateOptionsMenu(android.view.Menu menu){
    	super.onCreateOptionsMenu(menu);
    	menu.add(0,0,0,"Atualizar Lista");
    	menu.add(0,1,0,"Salvar Nomes");
		return true;
    }

    @Override
    public boolean onOptionsItemSelected(MenuItem item){
    	super.onOptionsItemSelected(item);
    	switch(item.getItemId()){
    		case  0: this.updateAdapter();
    			     break;
    		
    		case  1: this.salvarNomes(); 
    	}
    	
    	return true;
    }

    /**
     * Método executado quando o usuário arrasta alguma SeekBar.
     * Envia o estado da SeekBar arrastada (0-100) para o SAR como 
     * valor de estado da lâmpada.
     */
	@Override
	public void onProgressChanged(SeekBar seekBar, int progress, boolean fromUser) {
		// verifica se a mudança de progresso foi causada pelo usuário
		if (fromUser == true) {

			seekPosition = lista.getPositionForView(seekBar);
			
			// utilizado para diminuir a quantidade de mensagens enviadas em sequência
			int limite = limiares.get(seekPosition);
			CheckBox box = (CheckBox)adapter.getBoxes().get(seekPosition);
			box.setChecked(true);
			
			seekResposnse = conexao.enviaMensagem(Conexao.MSG_CONTROLE, enderecos.get(seekPosition), ""+(progress*255/100), disps.get(seekPosition));

			if ((progress == 0) && (limite != 0)) {

				box.setBackgroundResource(R.drawable.lampapagada);
				box.setChecked(false);
				limiares.set(seekPosition, 0);
			} else if ((progress > 0) && (progress < 25) && (limite != 1)) {

				box.setBackgroundResource(R.drawable.lampacesa25);
				limiares.set(seekPosition, 1);
			} else if ((progress >= 25) && (progress < 50) && (limite != 2)) {

				box.setBackgroundResource(R.drawable.lampacesa50);
				limiares.set(seekPosition, 2);
			} else if ((progress >= 50) && (progress < 75) && (limite != 3)) {

				box.setBackgroundResource(R.drawable.lampacesa75);
				limiares.set(seekPosition, 3);
			} else if ((progress >= 75) && (progress < 100) && (limite != 4)) {

				box.setBackgroundResource(R.drawable.lampacesa100);
				limiares.set(seekPosition, 4);
			} else if ((progress == 100) && (limite != 5)) {

				box.setBackgroundResource(R.drawable.lampacesa100);
				limiares.set(seekPosition, 5);
			}
			
			estados.set(seekPosition, ""+(progress*255/100));
		}
	}

	@Override
	public void onStartTrackingTouch(SeekBar seekBar) {
		// TODO Auto-generated method stub
	}

	/**
	 * Método invocado quando o usuário para de tocar em alguma SeekBar.
	 * Analisa a resposta do SAR e configura a cor do texto com o nome da 
	 * lâmpada dependendo da resposta recebida.
	 */
	@Override
	public void onStopTrackingTouch(SeekBar seekBar) {
		if (seekResposnse.equals(Conexao.MSG_ERRO)) {
			((TextView)adapter.getLabels().get(seekPosition)).setTextColor(Color.RED);
			adapter.getCores().set(seekPosition, Color.RED);
		}else{
			((TextView)adapter.getLabels().get(seekPosition)).setTextColor(Color.WHITE);
			adapter.getCores().set(seekPosition, Color.WHITE);
		}
	}

	/**
	 * Método invocado quando algum CheckBox (imagem de lâmpada) for tocado.
	 * Alterna estado da lâmpada entre 0 e 100 porcento e o valor para o SAR.
	 */
	@Override
	public void onClick(View arg0) {
		
		CheckBox box = (CheckBox)arg0;
		int i = lista.getPositionForView(box);
		String response = "";
		int valor;
		int limite;
		int prog;

		if (box.isChecked()) {
			valor = 255;
			limite = 5;
			prog = 100;
			box.setBackgroundResource(R.drawable.lampacesa100);
		} else {
			valor = 0;
			limite = 0;
			prog = 0;
			box.setBackgroundResource(R.drawable.lampapagada);
		}	
		
		response = conexao.enviaMensagem(Conexao.MSG_CONTROLE, enderecos.get(i), ""+valor, disps.get(i));
		
		if (response.equals("") || response.equals(Conexao.MSG_ERRO)) {
			((TextView)adapter.getLabels().get(i)).setTextColor(Color.RED);
			adapter.getCores().set(i, Color.RED);
		}else{
			((TextView)adapter.getLabels().get(i)).setTextColor(Color.WHITE);
			adapter.getCores().set(i, Color.WHITE);
		}
		((SeekBar)adapter.getBarras().get(i)).setProgress(prog);
		estados.set(i, ""+valor);
		limiares.set(i, limite);
	}
	
	/**
	 * Requisita ao SAR a lista de lâmpadas ativas na rede, com essas informações atualiza a ListView.
	 */
	public void updateAdapter(){
		
		String base = conexao.enviaMensagem(Conexao.REQUEST_TIPO, "0", "100", "0");
		if(base.equals("")){
			criarDialog(false);
			return;
		}
		
		enderecos.clear();
		nomes.clear();
		estados.clear();
		disps.clear();
		
		limiares.clear();
		
		if (!base.equals("null")) {

			String[] partes = base.split("#");

			for (String temp : partes) {

				String[] inf = temp.split("%", 2);

				if (inf.length == 2) {
					String endereco = inf[0];

					inf = inf[1].split("%");

					for (String s : inf) {

						String[] vetor = s.split(":");

						enderecos.add(endereco);
						nomes.add(vetor[0]);
						disps.add(vetor[1]);
						estados.add(vetor[2]);

						limiares.add(-1);
					}
				}
			}
		}
		
		adapter = new LampadaAdapter(this);			
		lista.setAdapter(adapter);
			
	}
	
	/**
	 * Envia lista de nomes das lâmpadas para o SAR.
	 */
	public void salvarNomes(){
		
		String msg = "";
		String antigoAddr = "";
		
		if (enderecos.size() > 0) {
			if (enderecos.size() == nomes.size()) {
				for (int i = 0; i < enderecos.size(); i++) {
					if (enderecos.get(i).equals(antigoAddr)) {
						msg += "#" + nomes.get(i);
					} else {
						msg += "%" + enderecos.get(i) + "#" + nomes.get(i);
					}
					antigoAddr = enderecos.get(i);
				}
			}
			mudou = false;
		}
		
		if (!msg.equals("")) {
			String base = conexao.enviaMensagem(Conexao.MSG_SET_NOME, "0",
					msg.substring(1), "0");
			if (base.equals("")) {
				criarDialog(false);
			}
			if (base.equals(Conexao.MSG_ERRO)) {
				Toast.makeText(this, "Permissão negada!", Toast.LENGTH_LONG)
						.show();
			}
			if (base.equals(Conexao.MSG_SUCESSO)) {
				Toast.makeText(this, "Nomes atualizados!", Toast.LENGTH_SHORT)
						.show();
			}
		}
		
	}
	
	/**
	 * Configura a imagem de lâmpada de uma determinada CheckBox de acordo o valor do parametro progresso. 
	 * @param box CheckBox onde será alterada a imagem
	 * @param progresso valor da luminosidade da lâmpada obtido da SeekBar
	 */
	public void setImagem(CheckBox box, int progresso){
		
		box.setChecked(true);
		
		if(progresso == 0){
			box.setBackgroundResource(R.drawable.lampapagada);
			box.setChecked(false);
		}else if((progresso > 0) && (progresso < 25)){
			box.setBackgroundResource(R.drawable.lampacesa25);
		}else if((progresso >= 25) && (progresso < 50)){
			box.setBackgroundResource(R.drawable.lampacesa50);
		}else if((progresso >= 50) && (progresso < 75)){
			box.setBackgroundResource(R.drawable.lampacesa75);
		}else{
			box.setBackgroundResource(R.drawable.lampacesa100);
		}
		
	}
	
	/**
	 * Recupera as informações necessárias para a conexão com o SAR.
	 * @return true se as informações foram obtidas com sucesso
	 */
	public boolean setInfoServidor(){
    	try {
			FileInputStream fIn = openFileInput("conf.dat");
			Scanner file_scan = new Scanner(fIn);

			if (file_scan.hasNext()) {
				usuario = file_scan.nextLine();
			}
			if (file_scan.hasNext()) {
				senha = file_scan.nextLine();
			}
			if (file_scan.hasNext()) {
				ip = file_scan.nextLine();
			}
			if (file_scan.hasNext()) {
				porta = file_scan.nextLine();
			}
			if (file_scan.hasNext()) {
				if (file_scan.nextLine().equals("true")) {
					isHttp = true;
				} else {
					isHttp = false;
				}
			}
			
			file_scan.close();
			fIn.close();

		} catch (IOException ioe) {		
			ioe.printStackTrace();		
			return false;
		}	
		
		return true;
    }
	
	/**
	 * Cria dialog para exibir mensagens de erro.
	 * @param isAutErro tipo de mensagem de erro exibida.
	 */
	public void criarDialog(boolean isAutErro){
		
		String mensagem = "";
		
		if(isAutErro == true){
			mensagem = "Erro na autenticação";
		}else{
			mensagem = "Erro na conexão com o servidor!";
		}
		
    	AlertDialog dialog = new AlertDialog.Builder(this).create();
		dialog.setButton("OK", new DialogInterface.OnClickListener(){
			public void onClick(DialogInterface dialog, int whichButton){
				Lampadas.this.finish();
			}
		});
		dialog.setTitle("Erro");
		dialog.setMessage(mensagem);
		dialog.show();
    }
	
	/**
	 * @return lista com os nomes das lâmpadas
	 */
	public ArrayList<String> getNomes(){
		return nomes;
	}
	
	/**
	 * @return lista com os estados das lâmpadas
	 */
	public ArrayList<String> getEstados(){
		return estados;
	}

	/**
	 * Método invocado quando o usuário tocar no nome de alguma lâmpada.
	 * Cria dialog para que o usuário possa alterar o nome da lâmpada tocada.
	 */
	@Override
	public boolean onTouch(final View textView, MotionEvent arg1) {
		
		final int position = lista.getPositionForView(textView);
							
		LayoutInflater layoutInflater = (LayoutInflater)getSystemService(LAYOUT_INFLATER_SERVICE);
		View view = layoutInflater.inflate(R.layout.edit_nome, null);
			
		final EditText etNome = (EditText)view.findViewById(R.id.etEditNome);
			
		AlertDialog dialogEditNome = new AlertDialog.Builder(this).create();
		dialogEditNome.setButton("Alterar", new DialogInterface.OnClickListener(){
			public void onClick(DialogInterface dialog, int whichButton){
				((TextView)textView).setText(etNome.getText().toString());
				nomes.set(position, etNome.getText().toString());
				mudou = true;
			}
		});
		dialogEditNome.setButton2("Cancelar", new DialogInterface.OnClickListener(){
			public void onClick(DialogInterface dialog, int whichButton){}
		});
		dialogEditNome.setTitle("Editar Nome");
		dialogEditNome.setView(view);
		
		etNome.setText(nomes.get(position));		
		dialogEditNome.show();
		
		return false;
	}

	/**
	 * Cria dialog com a opção enviar a lista com os nomes das
	 * lâmpadas para o SAR caso o botão 'voltar' tenha sido pressionado 
	 * e alguma alteração tenha sido feita.
	 */
	public boolean onKeyDown (int keyCode, KeyEvent event){
    	if((keyCode == KeyEvent.KEYCODE_BACK) && (mudou == true)){
    		AlertDialog dialog = new AlertDialog.Builder(this).create();
    		dialog.setButton("Sim", new DialogInterface.OnClickListener(){
    			public void onClick(DialogInterface dialog, int whichButton){		 					
    				salvarNomes();
    				finish();
    			}
    		});
    		dialog.setButton2("Não", new DialogInterface.OnClickListener(){
    			public void onClick(DialogInterface dialog, int whichButton){
    				finish();
    			}
    		});
    		dialog.setTitle("Salvar Nomes");
    		dialog.setMessage("Você deseja salvar as alterações?");
    		dialog.show();
    		return false;
    	}else{   	
    		return super.onKeyDown(keyCode, event);
    	}
    }

}