/**
 *  Copyright (C) 2009 Michel Vinicius de Melo Euzebio e Emerson Ribeiro de Mello
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 *  Copyright (C) 2009 Michel Vinicius de Melo Euzebio e Emerson Ribeiro de Mello
 *
 *  Este programa é software livre; você pode redistribuí-lo e/ou
 *  modificá-lo sob os termos da Licença Pública Geral GNU, conforme
 *  publicada pela Free Software Foundation; tanto a versão 2 da
 *  Licença como (a seu critério) qualquer versão mais nova.
 *
 *  Este programa é distribuído na expectativa de ser útil, mas SEM
 *  QUALQUER GARANTIA; sem mesmo a garantia implícita de
 *  COMERCIALIZAÇÃO ou de ADEQUAÇÃO A QUALQUER PROPÓSITO EM
 *  PARTICULAR. Consulte a Licença Pública Geral GNU para obter mais
 *  detalhes.
 *
 *  Você deve ter recebido uma cópia da Licença Pública Geral GNU
 *  junto com este programa; se não, escreva para a Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
 *  02111-1307, USA.
*/

package app.dl;

import android.content.Context;
import android.graphics.Color;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.BaseExpandableListAdapter;
import android.widget.CheckBox;
import android.widget.CompoundButton;
import android.widget.CompoundButton.OnCheckedChangeListener;
import android.widget.ImageView;
import android.widget.TextView;
import app.dl.InfoPerf;

/**
 * Classe que monta a lista de views exibida pela ExpandableListView a 
 * partir das informações dos perfis criados.
 */
public class PerfisAdapter extends BaseExpandableListAdapter {
	    
    private final int paddingGroup = 36;
    private final int paddingChild = 53;
    private boolean pVez;
    
    private LayoutInflater mInflater;
	private Perfis pai;
    
    public PerfisAdapter(Context context, boolean pv) {
		mInflater = LayoutInflater.from(context);
		pai = (Perfis) context;		
		pVez = pv;
	}

    public Object getChild(int groupPosition, int childPosition) {
        return pai.getInfoPerf().get(groupPosition).nomeDisp.get(childPosition);
    }

    public long getChildId(int groupPosition, int childPosition) {
        return childPosition;
    }

    /**
     * @return quantidade de subitens que serão exibidos por um determido perfil
     */
    public int getChildrenCount(int groupPosition) {
        return pai.getInfoPerf().get(groupPosition).endereco.size();
    }

    /**
	 * Método invocado a cada atualização de tela que retorna a View 
	 * que será exibida em cada posição da ExpandableListView.
	 * @param item objeto contendo as informações dos dispositivos pertencentes aos perfis
	 * @param groupPosition posição do item de perfil na lista em que o subitem será exibido
     * @param childPosition posição em que o subitem será exibido no item de perfil
	 * @param isChild true se a View retornada é um item (perfil) ou subitem (informação de dispositivo)
	 * @return View que será exibida em uma determinada posição da ExpandableListView
	 */
    public View getGenericView( Object item, final int groupPosition, final int childPosition, final boolean isChild ) {
    	
    	View xml = mInflater.inflate(R.layout.itens_lista_layout, null);
    	View root = xml.findViewById(R.id.root);
    	
    	TextView textoTitulo = (TextView)xml.findViewById(R.id.itemListaTitulo);
    	TextView textoSubtitulo = (TextView)xml.findViewById(R.id.itemListaSubtitulo);
    	CheckBox enable = (CheckBox)xml.findViewById(R.id.enableCheckBox); 	
    	
    	enable.setOnCheckedChangeListener(new OnCheckedChangeListener(){
			@Override
			public void onCheckedChanged(CompoundButton buttonView, boolean isChecked) {
				if(isChild == true){
					pai.getInfoPerf().get(groupPosition).enableDisp.set(childPosition, isChecked);
				}else{
					pai.getEnablePerfil().set(groupPosition, isChecked);
				}
				pai.setMudou(true);
			}   		
    	});
    	
    	String titulo = "";
    	String subtitulo = "";
    	
    	if(isChild == true){
    		
    		InfoPerf infoPerf = ((InfoPerf)item);
    		
    		String ini_hora = infoPerf.ini_hora.get(childPosition);
    		String ini_min = infoPerf.ini_min.get(childPosition);
    		String fim_hora = infoPerf.fim_hora.get(childPosition);
    		String fim_min = infoPerf.fim_min.get(childPosition);
    		String ini_valor = infoPerf.valorInicial.get(childPosition);
    		String fim_valor = infoPerf.valorFinal.get(childPosition);
    		
    		titulo = infoPerf.nomeDisp.get(childPosition);
    		subtitulo = ini_hora+":"+ini_min+" - "+fim_hora+":"+fim_min+", "+ini_valor+"%"+" - "+fim_valor+"%";
    		enable.setChecked(pai.getInfoPerf().get(groupPosition).enableDisp.get(childPosition));
    		
    		root.setBackgroundColor(Color.argb(100, 40, 40, 40));
    		textoTitulo.setPadding(paddingChild, 0, 0, 0);  
        	textoSubtitulo.setPadding(paddingChild, 0, 0, 0);  
    		((ImageView)xml.findViewById(R.id.itemListaIcone)).setBackgroundResource(R.drawable.lampapagada);
    	
    	}else{
    		
    		String[] dias = pai.getDiasPerfil().get(groupPosition);
    		String strTemp = "";
    		
    		if(dias[0].equals("-1")){		
    			strTemp = dias[1] + " de " + pai.meses[Integer.parseInt(dias[2])] + " de " + dias[3];
    		}else{
    			for(String s : dias){
        			strTemp += ", "+pai.dias_da_semana[Integer.parseInt(s) - 1].substring(0,3);
        		}
    			strTemp = strTemp.substring(2);
    		}
    		
    		titulo = (String) getGroup(groupPosition);
    		subtitulo = strTemp;
    		enable.setChecked(pai.getEnablePerfil().get(groupPosition));
    		
    		root.setPadding(paddingGroup, 0, 0, 0); 
    	}
    	
    	if(pVez == true){
    		pai.setMudou(false);
    	}
    	textoTitulo.setText(titulo);
    	textoSubtitulo.setText(subtitulo);
        
        return root;
    }
    
    /**
     * @param groupPosition posição do item de perfil na lista em que o subitem será exibido
     * @param childPosition posição em que o subitem será exibido no item de perfil
     * @param isLastChild se o subitem será exibido na ultima posição no item de perfil
     * @param convertView View (informação de um dispositivo) que será exibida no item de perfil
     * @param parent ExpandableListView onde serão exibidos os itens
     * @return subitem (informação de um dispositivo) que será exibido por um determido item de perfil
     */
    public View getChildView(int groupPosition, int childPosition, boolean isLastChild,
            View convertView, ViewGroup parent) {
    	return getGenericView(pai.getInfoPerf().get(groupPosition), groupPosition, childPosition, true);
    }

    public Object getGroup(int groupPosition) {
        return pai.getNomePerfil().get(groupPosition);
    }

    /**
     * @return quantidade de itens (informações de perfis) que serão exibidos
     */
    public int getGroupCount() {
        return pai.getNomePerfil().size();
    }

    public long getGroupId(int groupPosition) {
        return groupPosition;
    }

    /**
     * @param groupPosition posição da lista em que o perfil será exibido
     * @param isExpanded true se o item de perfil está atualmente expandido
     * @param convertView View (item) que será exibida na ListView
     * @param parent ExpandableListView onde serão exibidos os itens
     * @return item (informações de perfil) que será exibido na ExpandableListView
     */
    public View getGroupView(int groupPosition, boolean isExpanded, View convertView,
            ViewGroup parent) {    
    	return getGenericView(null, groupPosition, 0, false);
    }

    public boolean isChildSelectable(int groupPosition, int childPosition) {
        return true;
    }

    public boolean hasStableIds() {
        return true;
    }
}
