/**
 *  Copyright (C) 2009 Michel Vinicius de Melo Euzebio e Emerson Ribeiro de Mello
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 *  Copyright (C) 2009 Michel Vinicius de Melo Euzebio e Emerson Ribeiro de Mello
 *
 *  Este programa é software livre; você pode redistribuí-lo e/ou
 *  modificá-lo sob os termos da Licença Pública Geral GNU, conforme
 *  publicada pela Free Software Foundation; tanto a versão 2 da
 *  Licença como (a seu critério) qualquer versão mais nova.
 *
 *  Este programa é distribuído na expectativa de ser útil, mas SEM
 *  QUALQUER GARANTIA; sem mesmo a garantia implícita de
 *  COMERCIALIZAÇÃO ou de ADEQUAÇÃO A QUALQUER PROPÓSITO EM
 *  PARTICULAR. Consulte a Licença Pública Geral GNU para obter mais
 *  detalhes.
 *
 *  Você deve ter recebido uma cópia da Licença Pública Geral GNU
 *  junto com este programa; se não, escreva para a Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
 *  02111-1307, USA.
*/

package app.dl;

import java.io.FileInputStream;
import java.io.IOException;
import java.util.Scanner;

import android.app.Activity;
import android.app.AlertDialog;
import android.content.DialogInterface;
import android.content.Intent;
import android.os.Bundle;
import android.text.Html;
import android.view.LayoutInflater;
import android.view.MenuItem;
import android.view.View;
import android.view.View.OnClickListener;
import android.widget.Button;
import android.widget.EditText;
import android.widget.TextView;
import android.widget.Toast;

/**
 * Classe responsável pela criação do menu principal e do menu de opções 
 * para se acessar as configurações de varredura.
 */
public class Principal extends Activity implements OnClickListener{
	
	private Conexao conexao;
	private String usuario;
    private String senha;
    private String ip;
    private String porta;
    private boolean isHttp;
    private AlertDialog dialogVar;
	
	@Override
    public void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);       
        setContentView(R.layout.principal);
	
		autenticar();
        
		Button midiaBtn = (Button) findViewById(R.id.botaoMidia);
		Button lampadaBtn = (Button) findViewById(R.id.botaoLampada);
		Button perfisBtn = (Button) findViewById(R.id.botaoPerfis);
		Button configBtn = (Button) findViewById(R.id.botaoConfig);

		midiaBtn.setOnClickListener(this);
		lampadaBtn.setOnClickListener(this);
		perfisBtn.setOnClickListener(this);
		configBtn.setOnClickListener(this);
        
	}
	
	/**
	 * Envia mensagem de autenticação ao SAR caso já tenham sido feitas as configurações, 
	 * se não, inicia a Activity de configurações
	 */
	public void autenticar(){
		// configura atributos para conexao
		if (setInfoServidor() == true) {

			conexao = new Conexao(this, ip, porta, isHttp);
			String response = conexao.enviaMensagem(Conexao.MSG_AUTENTICACAO, usuario, senha, "");

			if (response.equals(Conexao.MSG_ERRO)) {
				criarDialog(true);
			}
			if (response.equals(Conexao.MSG_SUCESSO)) {
				Toast.makeText(this, "Conexão efetuada com sucesso!", Toast.LENGTH_SHORT).show();
			} else if (response.equals("")) {
				criarDialog(false);
			}
		}else{
			startActivity(new Intent(this, Configuracoes.class));
		}
	}
	
	@Override
    public boolean  onCreateOptionsMenu(android.view.Menu menu){
    	super.onCreateOptionsMenu(menu);
    	menu.add(0,0,0,"Fazer Varredura");
    	menu.add(0,1,0,"Autenticar");
    	menu.add(0,2,0,"Varredura Periódica");
		return true;
    }

    @Override
    public boolean onOptionsItemSelected(MenuItem item){
    	super.onOptionsItemSelected(item);
    	switch(item.getItemId()){
    		case  0:   

    			// envia mensagem de varredura ao SAR
    			try {
    				openFileInput("conf.dat");
    				    				
    				String response = conexao.enviaMensagem(Conexao.MSG_VARREDURA, "0", "0", "0");

					if (response.equals("")) {
						criarDialog(false);
					}else if (response.equals(Conexao.MSG_ERRO)) {
						criarDialog(true);
					}else{
						Toast.makeText(Principal.this, "Varredura executada com sucesso!", Toast.LENGTH_SHORT).show();
					}
 
    			} catch (Exception ioe) {
    				Intent i = new Intent(this, Configuracoes.class);
    				startActivity(i);
    			}
    			break;
    			
    		case  1: autenticar();
    				 break;
    				 
    		case 2: 
    			
    			try {
    				openFileInput("conf.dat");		
    				varDialog();

    			} catch (Exception ioe) {
    				Intent i = new Intent(this, Configuracoes.class);
    				startActivity(i);
    			}
 
    	}
    	
    	return true;
    }
    
    /**
     * Cria dialog para configurar a varredura, também envia mensagens 
     * de ativação ou desativação da varredura ao SAR
     */
    public void varDialog(){
    	
    	if(dialogVar == null){
    	
    		LayoutInflater layoutInflater = (LayoutInflater)getSystemService(LAYOUT_INFLATER_SERVICE);
    		View view = layoutInflater.inflate(R.layout.edit_var, null);
			
    		final EditText time = (EditText)view.findViewById(R.id.etSleepTime);
			
    		dialogVar = new AlertDialog.Builder(this).create();
    		dialogVar.setButton("Ativar", new DialogInterface.OnClickListener(){
    			public void onClick(DialogInterface dialog, int whichButton){
    				if(!time.getText().toString().equals("")){
    					String response = conexao.enviaMensagem(Conexao.SET_VARREDURA, "0", "1:"+time.getText().toString(), "0");
    					
    					if (response.equals(Conexao.MSG_ERRO)) {
    						Toast.makeText(Principal.this, "Permissão negada!", Toast.LENGTH_SHORT).show();
    					} if(response.equals(Conexao.MSG_SUCESSO)){
    						Toast.makeText(Principal.this, "Varredura ativada com sucesso!", Toast.LENGTH_SHORT).show();
    					}else if (response.equals("")) {
    						Toast.makeText(Principal.this, "Erro na conexão com o servidor!", Toast.LENGTH_SHORT).show();
    					}
    				}
    			}
    		});
    		dialogVar.setButton2("Desativar", new DialogInterface.OnClickListener(){
    			public void onClick(DialogInterface dialog, int whichButton){
    				String response = conexao.enviaMensagem(Conexao.SET_VARREDURA, "0", "0:0", "0");
    				
    				if (response.equals(Conexao.MSG_ERRO)) {
						Toast.makeText(Principal.this, "Permissão negada!", Toast.LENGTH_SHORT).show();
					} if(response.equals(Conexao.MSG_SUCESSO)){
						Toast.makeText(Principal.this, "Varredura desativada com sucesso!", Toast.LENGTH_SHORT).show();
					}else if (response.equals("")) {
						Toast.makeText(Principal.this, "Erro na conexão com o servidor!", Toast.LENGTH_SHORT).show();
					}
    			}
    		});
    		TextView label = (TextView)view.findViewById(R.id.tvVarLabel);
    		label.setText(Html.fromHtml("Intervalo de tempo entre varreduras<br>(em minutos)"));
    		
    		dialogVar.setTitle("Editar Varredura");
    		dialogVar.setMessage("Você deseja ativar ou desativar a Varredura Periódica?");
    		dialogVar.setView(view);
		
    		time.setText("15");	
    	}
		dialogVar.show();
    }

    /**
     * Inicia determinada Activity dependendo do botão clicado
     */
	@Override
	public void onClick(View v) {
		Intent i = null;
		int requestCode = 0;

		switch (v.getId()) {
		case R.id.botaoMidia:
			break;

		case R.id.botaoLampada:
			i = new Intent(this, Lampadas.class);
			break;

		case R.id.botaoPerfis:
			i = new Intent(this, Perfis.class);
			break;

		case R.id.botaoConfig:
			i = new Intent(this, Configuracoes.class);
			requestCode = 1;
		}	
		
		if(i != null){
			startActivityForResult(i, requestCode);
		}		
	}
	
	/**
	 * Configura os atributos utilizados para a conexão com o SAR a partir de
	 * informações lidas no arquivo de configurações
	 * @return
	 */
	public boolean setInfoServidor(){
    	try {
			FileInputStream fIn = openFileInput("conf.dat");
			Scanner file_scan = new Scanner(fIn);

			if (file_scan.hasNext()) {
				usuario = file_scan.nextLine();
			}
			if (file_scan.hasNext()) {
				senha = file_scan.nextLine();
			}
			if (file_scan.hasNext()) {
				ip = file_scan.nextLine();
			}
			if (file_scan.hasNext()) {
				porta = file_scan.nextLine();
			}
			if (file_scan.hasNext()) {
				if (file_scan.nextLine().equals("true")) {
					isHttp = true;
				} else {
					isHttp = false;
				}
			}
			
			file_scan.close();
			fIn.close();

		} catch (IOException ioe) {
			ioe.printStackTrace();	
			return false;
		}	
		return true;
    }

	/**
	 * Cria dialog para as mensagens de erro
	 * @param isAutErro determina o tipo de mensagem a ser exibida
	 */
	public void criarDialog(boolean isAutErro){
		
		String mensagem = "";
		
		if(isAutErro == true){
			mensagem = "Erro na autenticação";
		}else{
			mensagem = "Erro na conexão com o servidor!";
		}
		
    	AlertDialog dialog = new AlertDialog.Builder(this).create();
		dialog.setButton("OK", new DialogInterface.OnClickListener(){
			public void onClick(DialogInterface dialog, int whichButton){}
		});
		dialog.setTitle("Erro");
		dialog.setMessage(mensagem);
		dialog.show();
    }
	
	/**
	 * É executado quando a aplicação retorna da Activity de configurações, 
	 * então é feita uma nova autenticação no SAR com os dados atualizados
	 */
	@Override
	protected void onActivityResult(int requestCode, int resultCode, Intent data) {
	    super.onActivityResult(requestCode, resultCode, data);
	    
	    if((requestCode == 1) && (resultCode != Activity.RESULT_CANCELED)){
	    	autenticar();
	    }
	}

}
